package com.yfd.platform.modules.specialDocument.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yfd.platform.config.ResponseResult;
import com.yfd.platform.exception.file.InvalidStorageSourceException;
import com.yfd.platform.modules.config.model.request.FileListRequest;
import com.yfd.platform.modules.specialDocument.domain.Files;
import com.yfd.platform.modules.specialDocument.mapper.FilesMapper;
import com.yfd.platform.modules.specialDocument.service.IFilesService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.storage.chain.FileChain;
import com.yfd.platform.modules.storage.chain.FileContext;
import com.yfd.platform.modules.storage.context.StorageSourceContext;
import com.yfd.platform.modules.storage.controller.file.FileController;
import com.yfd.platform.modules.storage.model.enums.FileTypeEnum;
import com.yfd.platform.modules.storage.model.request.BatchDeleteRequest;
import com.yfd.platform.modules.storage.model.request.RenameFileRequest;
import com.yfd.platform.modules.storage.model.result.FileInfoResult;
import com.yfd.platform.modules.storage.model.result.FileItemResult;
import com.yfd.platform.modules.storage.service.StorageSourceService;
import com.yfd.platform.modules.storage.service.base.AbstractBaseFileService;
import com.yfd.platform.system.domain.LoginUser;
import com.yfd.platform.utils.StringUtils;
import io.netty.channel.ChannelInboundHandlerAdapter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 * 专项文档表 服务实现类
 * </p>
 *
 * @author LiMengNan
 * @since 2025-01-20
 */
@Service
public class FilesServiceImpl extends ServiceImpl<FilesMapper, Files> implements IFilesService {
    private static final Logger LOGGER = LoggerFactory.getLogger(ChannelInboundHandlerAdapter.class);

    //专项文档表 Mapper
    @Resource
    private FilesMapper filesMapper;

    @Autowired
    private FileController fileController;

    @Resource
    private StorageSourceContext storageSourceContext;

    @Resource
    private StorageSourceService storageSourceService;

    @Resource
    private FileChain fileChain;


    /**********************************
     * 用途说明: 分页查询专项文档管理-文档内容
     * 参数说明
     * fileName      文件名称
     * startDate     (开始日期)
     * endDate      （结束日期）
     * keywords      关键字
     *nodeId         节点ID
     *projectId      所属项目ID
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Override
    public Page<Files> getFilesPage(String fileName, String startDate, String endDate, String keywords, String nodeId, String projectId, String fileName1, Page<Files> page) throws Exception {
        //先查询路径下的所有文件
        //首先通过项目ID  和节点ID去查询表   获取一个路径  如果不是空 就调用minio的获取文件列表接口    查询的数据放在集合中
        FileInfoResult fileInfoResult = null;

        QueryWrapper<Files> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("project_id", projectId);
        queryWrapper.eq("node_id", nodeId);
        List<Files> filess = filesMapper.selectList(queryWrapper);
        String filePath = null;
        if (filess != null && !filess.isEmpty()) {
            for (Files files : filess) {
                filePath = files.getFilePath();
                if (filePath != null && !filePath.isEmpty()) {
                    break; // 如果找到非空的 filePath，终止循环
                }
            }
        }
        if (filePath != null) {
            FileListRequest fileListRequest = new FileListRequest();
            fileListRequest.setOrderBy("time");
            fileListRequest.setOrderDirection("desc");
            fileListRequest.setPassword("");
            fileListRequest.setPath(filePath);
            fileListRequest.setStorageKey("minio");

            String storageKey = fileListRequest.getStorageKey();
            Integer storageId = storageSourceService.findIdByKey(storageKey);
            if (storageId == null) {
                throw new InvalidStorageSourceException("通过存储源 key 未找到存储源, key: " + storageKey);
            }
            // 处理请求参数默认值
            fileListRequest.handleDefaultValue();
            // 获取文件列表
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageId(storageId);
            List<FileItemResult> fileItemList = fileService.fileList(fileListRequest.getPath());
            // 执行责任链
            FileContext fileContext = FileContext.builder()
                    .storageId(storageId)
                    .fileListRequest(fileListRequest)
                    .fileItemList(fileItemList).build();
            fileChain.execute(fileContext);
            fileInfoResult = new FileInfoResult(fileContext.getFileItemList(), fileContext.getPasswordPattern());
        }

        //分页查询专项文档管理-文档内容
        LambdaQueryWrapper<Files> queryWrapperfiles = new LambdaQueryWrapper<>();
        //如果文件名称  不为空
        if (StringUtils.isNotEmpty(fileName)) {
            queryWrapperfiles.like(Files::getFileName, fileName);
        }
        //如果关键字  不为空
        if (StringUtils.isNotEmpty(keywords)) {
            queryWrapperfiles.like(Files::getKeywords, keywords);
        }
        //开始时间startDate
        DateTime parseStartDate = DateUtil.parse(startDate);
        //时间endDate不为空
        DateTime parseEndDate = DateUtil.parse(endDate);
        //开始时间和结束时间不为空    查询条件>=开始时间  <结束时间
        if (parseStartDate != null && parseEndDate != null) {
            queryWrapperfiles.ge(Files::getUploadTime, parseStartDate).lt(Files::getUploadTime, parseEndDate);
        }
        queryWrapperfiles.eq(Files::getProjectId, projectId);//所属项目ID
        queryWrapperfiles.eq(Files::getNodeId, nodeId);//节点ID
        queryWrapperfiles.orderByDesc(Files::getUploadTime);//时间
        //分页查询
        Page<Files> filesPage = filesMapper.selectPage(page, queryWrapperfiles);
        //处理文件内容
        List<Files> records = filesPage.getRecords();
        for (Files files : records) {
            //循环从minio拿出来的数据  然后把url放到files里面
            files.setUrl("");
            files.setType("");
            List<FileItemResult> filelist = fileInfoResult.getFiles();
            if (filelist != null && !filelist.isEmpty()) {
                for (FileItemResult fileItemResult : filelist) {
                    if (fileItemResult.getUrl() == null || fileItemResult.getUrl().isEmpty()) {
                        continue;
                    }
                    if (files.getFileName().equals(fileItemResult.getName())) {
                        files.setUrl(fileItemResult.getUrl());
                        files.setType(fileItemResult.getType().getValue());
                        break;
                    }
                }

            }
        }
        filesPage.setRecords(records);
        return filesPage;
    }


    /***********************************
     * 用途说明：新增专项文档管理-文档内容
     * 参数说明
     * Files     文档内容
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Override
    public Boolean addFiles(Files files) {
        Boolean value = false;
        // 校验文件名是否包含非法字符
        String fileName = files.getFileName();
        if (containsInvalidCharacters(fileName)) {
            return value;
        }


        //  String[] splitIds = ids.split(",");   BigDecimal
        List<String> names = Arrays.asList(files.getFileName().split(","));
        List<String> sizes = Arrays.asList(files.getFileSize().split(","));

        //  差不多的流程  就是提出来  然后判断  如果两个 中有一个是true
        //获取当前登录用户         上传人是当前登录人
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();


        // 数据校验
        if (names.size() != sizes.size()) {
            LOGGER.error("文件名称和文件大小的列表长度不一致");
            return false;
        }
        List<Files> filesToSave = new ArrayList<>();
        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);
        files.setUploadTime(currentTime);

        for (int i = 0; i < names.size(); i++) {
            String name = names.get(i).trim();
            String sizeStr = sizes.get(i).trim();
            // 校验文件大小是否可以转换成数值
            try {
                Files files1 = new Files();
                files1.setProjectId(files.getProjectId());
                files1.setNodeId(files.getNodeId());
                files1.setFilePath(files.getFilePath());
                files1.setKeywords(files.getKeywords());
                files1.setDescription(files.getDescription());
                files1.setUploadTime(files.getUploadTime());
                files1.setUploader(loginuser.getUsername());
                files1.setFileName(name);
                files1.setFileSize(sizeStr);
                filesToSave.add(files1);
            } catch (NumberFormatException e) {
                LOGGER.error("文件大小必须是有效的数字");
            }
        }
        if(filesToSave.size()>0){
            //循环新增
            for(Files filess : filesToSave){
                int valueAdded = filesMapper.insert(filess);
                if (valueAdded == 1) {
                    value = true;
                } else {
                    value = false;
                }
            }
        }
        return value;
    }

    /**********************************
     * 用途说明: 修改专项文档管理-文档内容
     * 参数说明
     * Files     文档内容
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回修改成功或者失败
     ***********************************/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public boolean updateFiles(Files files) {
        // 校验文件名是否包含非法字符
        String fileName = files.getFileName();
        if (containsInvalidCharacters(fileName)) {
            return false;
        }

        // 修改之前查询表中的文件名是否修改，如果发生变动先修改 minio 然后再修改表结构
        Files filesData = filesMapper.selectById(files.getId());

        // 判断文件名是否修改
        if (!files.getFileName().equals(filesData.getFileName())) {
            // 修改数据库
            int valueUpdate = filesMapper.updateById(files);
            if (valueUpdate != 1) {
                LOGGER.error("表结构修改失败");
                throw new RuntimeException("更新数据库失败");
            }

            // 修改 MinIO 文件名
            boolean minioUpdateSuccess = updateMinioFileName(filesData, files);
            if (!minioUpdateSuccess) {
                // 如果 MinIO 修改失败，抛出异常，触发事务回滚
                LOGGER.error("表结构修改成功，MinIO 修改失败");
                throw new RuntimeException("在MinIO中重命名文件失败.");
            }
            LOGGER.info("MinIO 和表结构都修改成功");
            return true;

        } else {
            // 如果文件名没有修改，仅更新数据库
            int valueUpdate = filesMapper.updateById(files);
            return valueUpdate == 1;
        }
    }

    // 校验文件名是否包含非法字符
    private boolean containsInvalidCharacters(String fileName) {
        // 定义非法字符的正则表达式
        String regex = "[<>:\"/\\\\|?*]";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(fileName);
        return matcher.find();
    }

    /**********************************
     * 用途说明: 根据ID删除专项文档管理-文档内容
     * 参数说明  id 文档内容ID
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回删除成功或者失败
     **********************************
     * @return*/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public String deleteFilesByIds(List<String> dataset) {

        List<Files> filesList = filesMapper.selectBatchIds(dataset);

        int SuccessCount = 0, FailCount = 0, total = CollUtil.size(dataset);
        //Todo  最直接的办法   循环出来 一条一条删除
        for (Files files : filesList) {
            List<BatchDeleteRequest.DeleteItem> deleteItemList = new ArrayList<>();
            BatchDeleteRequest.DeleteItem deleteItemData = new BatchDeleteRequest.DeleteItem();
            deleteItemData.setName(files.getFileName());
            deleteItemData.setPassword("");
            deleteItemData.setPath(files.getFilePath());
            deleteItemData.setType(FileTypeEnum.FILE);
            deleteItemList.add(deleteItemData);

            //首先通过ID集合查询所有的内容  然后放到List<DeleteItem> deleteItems里面  放好以后删除数据库  然后删除minio

            BatchDeleteRequest batchDeleteRequest = new BatchDeleteRequest();
            batchDeleteRequest.setDeleteItems(deleteItemList);
            batchDeleteRequest.setStorageKey("minio");

            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(batchDeleteRequest.getStorageKey());
            List<BatchDeleteRequest.DeleteItem> deleteItems = batchDeleteRequest.getDeleteItems();

            int deleteSuccessCount = 0, deleteFailCount = 0, totalCount = CollUtil.size(deleteItems);
            for (BatchDeleteRequest.DeleteItem deleteItem : deleteItems) {

                boolean flag = false;
                try {
                    if (deleteItem.getType() == FileTypeEnum.FILE) {
                        flag = fileService.deleteFile(deleteItem.getPath(), deleteItem.getName());
                    } else if (deleteItem.getType() == FileTypeEnum.FOLDER) {
                        flag = fileService.deleteFolder(deleteItem.getPath(), deleteItem.getName());
                    }

                    if (flag) {
                        deleteSuccessCount++;
                    } else {
                        deleteFailCount++;
                    }
                } catch (Exception e) {
                    LOGGER.error("删除文件/文件夹失败, 文件路径: {}, 文件名称: {}", deleteItem.getPath(), deleteItem.getName(), e);
                    deleteFailCount++;
                }
            }

            if (totalCount > 1) {
                //return ResponseResult.success("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
            } else {
                //return totalCount == deleteSuccessCount ? ResponseResult.success("删除成功") : ResponseResult.error("删除失败");
                LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");

            }
            //如果是1 说明成功删除
            if (deleteSuccessCount == 1) {
                int valueDelete = filesMapper.deleteById(files.getId());
                if (valueDelete > 0) {
                    SuccessCount++;
                    LOGGER.info("表结构成功删除");
                } else {
                    System.out.println("没有记录被删除");
                }
            } else {
                FailCount++;
            }

        }
        return "批量删除 " + total + " 个, 删除成功 " + SuccessCount + " 个, 失败 " + FailCount + " 个.";
    }


    // 修改 MinIO 文件名的方法
    private boolean updateMinioFileName(Files filesData, Files files) {
        try {
            RenameFileRequest renameFileRequest = new RenameFileRequest();
            renameFileRequest.setName(filesData.getFileName());
            renameFileRequest.setNewName(files.getFileName());
            renameFileRequest.setPassword("");
            renameFileRequest.setPath(filesData.getFilePath());
            renameFileRequest.setStorageKey("minio");

            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(renameFileRequest.getStorageKey());
            return fileService.renameFile(renameFileRequest.getPath(), renameFileRequest.getName(), renameFileRequest.getNewName());
        } catch (Exception e) {
            LOGGER.error("MinIO 修改文件名时发生异常", e);
            return false;
        }
    }
}
